﻿using System;
using System.ComponentModel.Composition;

namespace AsynchronousExamples
{
    [Export(typeof(IMealsFacade))]
    public class FacadeImplementation : IMealsFacade
    {
        [Import]
        public IMeals Meals { get; set; }

        [Import]
        public MealProxy Proxy { get; set; }

        public void Breakfast(Action<bool> callback)
        {
            Meals.BeginBreakfast(EndBreakfast, callback);
        }

        private void EndBreakfast(IAsyncResult ar)
        {
            var callback = ar.AsyncState as Action<bool>;
            if (callback != null)
            {
                callback(Meals.EndBreakfast(ar));
            }
        }

        public void Lunch(Action<bool> callback)
        {
            Proxy.LunchCompleted += ProxyLunchCompleted;
            Proxy.LunchAsync(callback);
        }

        public void ProxyLunchCompleted(object sender, 
            MealProxy.MealCompletedEventArgs e)
        {
            Proxy.LunchCompleted -= ProxyLunchCompleted;
            var callback = e.UserState as Action<bool>;
            if (callback != null)
            {
                callback(e.Result);
            }
        }

        public void Dinner(Action<bool> callback)
        {
            throw new NotImplementedException();
        }
    }
}