﻿using System;
using System.ComponentModel;
using System.ComponentModel.Composition;

namespace AsynchronousExamples
{
    [Export]
    public class MealProxy : MealProcessor
    {
        public class MealCompletedEventArgs : AsyncCompletedEventArgs 
        {
            public MealCompletedEventArgs(
                Exception exception,
                bool cancelled,
                object userState) : 
                base(exception, cancelled, userState)
            {
                
            }

            public bool Result { get; set; }
        }

        public event EventHandler<MealCompletedEventArgs> BreakfastCompleted;
        public event EventHandler<MealCompletedEventArgs> LunchCompleted;
        public event EventHandler<MealCompletedEventArgs> DinnerCompleted;

        public void BreakfastAsync(object userState)
        {
            BeginLunch(EndBreakfastAsync, userState);
        }

        private void EndBreakfastAsync(IAsyncResult ar)
        {
            var completed = BreakfastCompleted;
            if (completed != null)
            {
                completed(this, new MealCompletedEventArgs(null, false, ar.AsyncState)
                {
                    Result = EndBreakfast(ar)
                });
            }
        }

        public void LunchAsync(object userState)
        {
            BeginLunch(EndLunchAsync, userState);
        }

        private void EndLunchAsync(IAsyncResult ar)
        {
            var completed = LunchCompleted;
            if (completed != null)
            {
                completed(this, new MealCompletedEventArgs(null, false, ar.AsyncState)
                                    {
                                        Result = EndLunch(ar)
                                    });
            }
        }

        public void DinnerAsync(object userState)
        {
            BeginLunch(EndDinnerAsync, userState);
        }

        private void EndDinnerAsync(IAsyncResult ar)
        {
            var completed = DinnerCompleted;
            if (completed != null)
            {
                completed(this, new MealCompletedEventArgs(null, false, ar.AsyncState)
                {
                    Result = EndDinner(ar)
                });
            }
        }
    }
}