﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Input;

namespace DataBindingExamples
{
    public class Country
    {
        public string Code { get; set; }
        public string Name { get; set; }      
    }

    public class State
    {
        public string Code { get; set; }
        public string Name { get; set; }

        public override string ToString()
        {
            return string.Format("{0} - {1}", Code, Name);
        }
    }

    public class Countries : INotifyPropertyChanged
    {
        public Countries()
        {
            ShowSelectedCountry = new ShowCountryCommand();
        }

        private readonly List<Country> _countries
            = new List<Country>
                    {
                        new Country
                            {
                                Code = "US",
                                Name = "United States"
                            },
                            new Country
                            {
                                Code = "CA",
                                Name = "Canada"
                            }
                    };

        private readonly Dictionary<string, List<State>> _states =
            new Dictionary<string, List<State>>
                {
                    {
                        "US", new List<State>
                                    {
                                        new State
                                            {
                                                Code = "FL", 
                                                Name = "Florida"
                                            },
                                        new State
                                            {
                                                Code = "GA", 
                                                Name = "Georgia"
                                            }
                                    }
                        },
                    {
                        "CA", new List<State>
                                    {
                                        new State
                                            {
                                                Code = "AB", 
                                                Name = "Alberta"
                                            },
                                        new State
                                            {
                                                Code = "BC", 
                                                Name = "British Columbia"
                                            }
                                    }
                        }
                };

        public List<Country> CountryList
        {
            get { return _countries; }
        }

        public List<State> StateList
        {
            get
            {
                return SelectedCountry == null ? 
                    new List<State>() : _states[SelectedCountry.Code];
            }
        }

        private Country _selectedCountry;

        public Country SelectedCountry
        {
            get { return _selectedCountry; }
            set
            {
                _selectedCountry = value;
                ((ShowCountryCommand)ShowSelectedCountry).RaiseCanExecuteChanged();                
                PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("SelectedCountry"));
                PropertyChanged(
                                this,
                                new PropertyChangedEventArgs("StateList"));
                SelectedState = StateList != null
                                    ? StateList[0]
                                    : null;
            }
        }

        public ICommand ShowSelectedCountry { get; private set; }

        private State _selectedState;

        public State SelectedState
        {
            get { return _selectedState; }
            set
            {
                _selectedState = value;
                PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("SelectedState"));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}