﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace DataBindingExamples
{
    public class MultiSelect : INotifyPropertyChanged
    {
        public string Value { get; set; }

        private bool _isSelected;

        public bool IsSelected
        {
            get { return _isSelected; }
            set 
            { 
                _isSelected = value;
                var handler = PropertyChanged;
                if (handler != null)
                {
                    handler(
                        this, 
                        new PropertyChangedEventArgs("IsSelected"));
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public override string ToString()
        {
            return Value;
        }
    }

    public class MultiSelection : INotifyPropertyChanged 
    {
        public MultiSelection()
        {
            Options = new List<MultiSelect>();
            for(var x = 0; x < 10; x++)
            {
                var option =
                    new MultiSelect
                        {
                            IsSelected = false,
                            Value = Guid.NewGuid().ToString()
                        };
                Options.Add(option);
                if (DesignerProperties.IsInDesignTool)
                {
                    return;
                }
                option.PropertyChanged
                    += (o, e) => 
                        {
                            if (!e.PropertyName.Equals("IsSelected"))
                            {
                                return;
                            }

                            if (PropertyChanged != null)
                            {
                                PropertyChanged(
                                    this,
                                    new PropertyChangedEventArgs("SelectedOptions"));
                            }
                        };               
            }
        }
        
        public List<MultiSelect> Options { get; private set; } 
     
        public IEnumerable<MultiSelect> SelectedOptions
        {
            get
            {
                if (DesignerProperties.IsInDesignTool)
                {
                    return (from o in Options
                            select o).Take(5);
                }
                return from o in Options
                        where o.IsSelected
                        select o;
            }
        }
  
        public event PropertyChangedEventHandler PropertyChanged;
    }
}