﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Media;

namespace DataBindingExamples
{
    public class MyThemes : INotifyPropertyChanged 
    {
        private const string DICTIONARY 
            = "/DataBindingExamples;component/Themes.xaml";

        private readonly ResourceDictionary _themeDictionary 
            = new ResourceDictionary();

        private int _colorIndex;

        private readonly List<SolidColorBrush> _colorBrushes 
            = new List<SolidColorBrush>();

        private int _gradientIndex;

        private readonly List<GradientBrush> _gradientBrushes 
            = new List<GradientBrush>();

        public SolidColorBrush SolidBrush
        {
            get { return _colorBrushes[_colorIndex]; }
        }

        public GradientBrush GradientBrush
        {
            get { return _gradientBrushes[_gradientIndex]; }
        }

        public MyThemes()
        {            
            _themeDictionary.Source = new Uri(DICTIONARY, UriKind.Relative);

            foreach (var item in from object key 
                                        in _themeDictionary.Keys 
                                    select _themeDictionary[key])
            {
                if (item is SolidColorBrush)
                {
                    _colorBrushes.Add(item as SolidColorBrush);
                }
                else if (item is GradientBrush)
                {
                    _gradientBrushes.Add(item as GradientBrush);
                }
            }
        }

        public void CycleTheme()
        {
            _colorIndex++;
            _colorIndex %= _colorBrushes.Count();
            _gradientIndex++;
            _gradientIndex %= _gradientBrushes.Count();
            RaisePropertyChange("SolidBrush");
            RaisePropertyChange("GradientBrush");
        }

        protected void RaisePropertyChange(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}