﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media.Animation;
using System.Xaml;

namespace DependencyProperties
{
    public class TextFlashExtension : MarkupExtension 
    {
        public string Text { get; set; }

        public TextFlashExtension()
        {
            Text = "This is sample text.";
        }

        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            var root = serviceProvider.GetService(typeof (IRootObjectProvider)) as IRootObjectProvider;
            var value = serviceProvider.GetService(typeof (IProvideValueTarget)) as IProvideValueTarget;

            if (root == null)
                return null;

            if (value == null)
                return null;

            // set up an animation that loops indefinitely 
            var control = root.RootObject as UserControl; 

            if (control != null)
            {
                var storyboard = new Storyboard();
                Storyboard.SetTarget(storyboard, 
                    value.TargetObject as FrameworkElement);
                Storyboard.SetTargetProperty(storyboard, 
                    new PropertyPath("(UIElement.Opacity)"));
                var anim = new DoubleAnimation
                                {
                                    Duration = TimeSpan.FromSeconds(1),
                                    RepeatBehavior = RepeatBehavior.Forever,
                                    AutoReverse = true,
                                    From = 0.1,
                                    To = 1.0
                                };
                storyboard.Children.Add(anim);
                var key = Guid.NewGuid().ToString();
                control.Resources.Add(key, storyboard);
                storyboard.Begin();
            }    
            
            return Text;
        }
    }
}