﻿using System;
using System.Collections.Generic;
using System.Json;
using System.Reflection;
using System.Reflection.Emit;

namespace DynamicDataExample
{
    public class DynamicTypeHelper
    {
        public Type DynamicType { get; private set; }

        public object Generate(JsonObject source)
        {
            var retVal = Activator.CreateInstance(DynamicType);
            foreach(var key in source.Keys)
            {
                SetValue(retVal, key, source[key].ToString());
            }
            return retVal;
        }

        private void SetValue(object target,
                             string propertyName, object value)
        {
            var setter = DynamicType.GetProperty(propertyName);
            setter.GetSetMethod().Invoke(target,
                                         new[] {value});
        }

        public DynamicTypeHelper(string typeName, IDictionary<string, JsonValue> template)
        {
            var name = new AssemblyName("DynamicTypeAssembly");
            var domain = AppDomain.CurrentDomain;
            var assemblyBuilder = domain
                .DefineDynamicAssembly(name,
                                        AssemblyBuilderAccess.Run);
            var builder = assemblyBuilder.DefineDynamicModule("DynamicModels");
            var typeBuilder = builder.DefineType(
                typeName,
                TypeAttributes.Class | TypeAttributes.Public);
            foreach(var propertyName in template.Keys)
            {
                var field = typeBuilder.DefineField(
                    string.Format("_{0}", propertyName),
                    typeof (string),
                    FieldAttributes.Private);
                var property = typeBuilder.DefineProperty(
                    propertyName, PropertyAttributes.HasDefault, typeof (string), null);

                var getter = typeBuilder.DefineMethod(string.Format("get_{0}", propertyName),
                    MethodAttributes.Public |
                    MethodAttributes.SpecialName |
                    MethodAttributes.HideBySig,
                    typeof(string), Type.EmptyTypes);
                var getterCode = getter.GetILGenerator();

                getterCode.Emit(OpCodes.Ldarg_0);
                getterCode.Emit(OpCodes.Ldfld, field);
                getterCode.Emit(OpCodes.Ret);

                var setter = typeBuilder.DefineMethod(string.Format("set_{0}", propertyName),
                    MethodAttributes.Public |
                    MethodAttributes.SpecialName |
                    MethodAttributes.HideBySig, null,
                        new[] { typeof(string) });

                var setterCode = setter.GetILGenerator();

                setterCode.Emit(OpCodes.Ldarg_0);
                setterCode.Emit(OpCodes.Ldarg_1);
                setterCode.Emit(OpCodes.Stfld, field);
                setterCode.Emit(OpCodes.Ret);

                property.SetGetMethod(getter);
                property.SetSetMethod(setter);
            }

            typeBuilder.DefineDefaultConstructor(MethodAttributes.Public);
            DynamicType = typeBuilder.CreateType();
        }
    }
}