using System;
using System.Collections.Generic;
using System.Json;
using System.Linq;

namespace DynamicDataExample
{
    public class JsonHelper 
    {
        private readonly IEnumerable<string> _keys = Enumerable.Empty<string>();
        private readonly Dictionary<string,Func<object,object>> _converters = new Dictionary<string, Func<object, object>>();

        public static JsonArray ParseAndLoad()
        {
            var json = typeof (JsonHelper).FullName.Replace(typeof(JsonHelper).Name,
                                                                     "Data.json");

            // configure the custom type 
            using (
                var stream =
                    typeof(CustomTypeViewModel).Assembly.GetManifestResourceStream(json))
            {
                var jsonArray = JsonValue.Load(stream) as JsonArray;
                return jsonArray;
            }            
        }      

        public JsonHelper(IDictionary<string,JsonValue> template)
        {
            _keys = (from k in template.Keys select k).ToArray();

            foreach (var key in template.Keys)
            {
                int integerTest;
                double doubleTest;
                var value = template[key].ToString();
                if (int.TryParse(value, out integerTest))
                {
                    CustomTypeHelper<CustomType>.AddProperty(key, typeof(int));
                    _converters.Add(key, obj => int.Parse(obj.ToString()));
                }
                else if (double.TryParse(value, out doubleTest))
                {
                    CustomTypeHelper<CustomType>.AddProperty(key, typeof(double));
                    _converters.Add(key, obj => double.Parse(obj.ToString()));
                }
                else
                {
                    CustomTypeHelper<CustomType>.AddProperty(key, typeof(string));                    
                    _converters.Add(key, obj =>
                                                {
                                                    // strip quotes
                                                    var str = obj.ToString().Substring(1);
                                                    return str.Substring(0, str.Length - 1);
                                                });
                }
            }
        }

        public void MapJsonObject(Action<string,object> setValue, JsonValue item)
        {            
            foreach (var key in _keys)
            {
                setValue(key, _converters[key](item[key]));
            }
        }
    }
}