﻿namespace FluentValidation
{
    public class IsIntegerInRange : IValidation
    {
        private readonly int _min;
        private readonly int _max = int.MaxValue;

        public IsIntegerInRange() { }
        
        public IsIntegerInRange(int min, int max)
        {
            _min = min;
            _max = max;
        }

        public bool Validate(string value)
        {
            if (string.IsNullOrEmpty(value))
            {
                return true;
            }

            int integer; 

            return int.TryParse(value, out integer) &&
                    integer >= _min && integer <= _max;
        }

        public string ValidationMessage
        {
            get
            {
                return string.Format(
                    "Field must be an integer between {0} and {1}",
                    _min,
                    _max);
            }

        }
    }
}