﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace FluentValidation
{
    public class ValidationHelper<T> : IValidationHelper 
        where T : BaseEntity         
    {
        private readonly T _instance;
        private readonly Dictionary<string, List<IValidation>> _validators
            = new Dictionary<string, List<IValidation>>();        

        public ValidationHelper(T instance)
        {
            _instance = instance;
            _instance.PropertyChanged += _instance_PropertyChanged;
        }

        void _instance_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (!_validators.ContainsKey(e.PropertyName))
            {
                return;
            }

            var value = _instance.GetType()
                .GetProperty(e.PropertyName)
                .GetGetMethod()
                .Invoke(_instance, null);

            var valueAsString = value == null
                                    ? string.Empty
                                    : value.ToString();

            _instance.ClearErrors(e.PropertyName);

            foreach (var validator in _validators[e.PropertyName]
                .Where(validator => !validator.Validate(valueAsString)))
            {
                _instance.SetError(
                    e.PropertyName, 
                    validator.ValidationMessage);
            }
        }

        public void RegisterValidation(
            string propertyName,
            IValidation validator)
        {
            if (!_validators.ContainsKey(propertyName))
            {
                _validators.Add(
                    propertyName,
                    new List<IValidation>());
            }

            _validators[propertyName].Add(validator);
        }
    }
}