﻿using System.ComponentModel;
using System.Diagnostics;

namespace HelloSilverlight.Model
{
    public class WelcomeContainer : INotifyPropertyChanged 
    {
        private static string _description;
        private static WelcomeContainer _instance;

        public static void SetDescription(string description)
        {
            _description = description;

            if (_instance != null)
            {
                _instance.RaisePropertyChange();
            }
        }

        public WelcomeContainer()
        {
            if (DesignerProperties.IsInDesignTool)
            {
                SetDescriptionInDesigner();
            }
            else
            {
                _instance = this;
            }
        }

        private string _descriptionField; 

        public string Description
        {
            get { return _description ?? _descriptionField; }
            set { _descriptionField = value; }
        }

        [Conditional("DEBUG")]
        private void SetDescriptionInDesigner()
        {
            Description = "This is designer-friendly data.";
        }

        private void RaisePropertyChange()
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs("Description"));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
