﻿using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Windows;
using System.Windows.Resources;
using System.Xml;

namespace HelloSilverlight
{
    public class XapDownloadService : IApplicationService 
    {
        public const string XAP_PARAM = "externalXap";
        public void StartService(ApplicationServiceContext context)
        {
            var xap = string.Empty;
            if (context.ApplicationInitParams.ContainsKey(XAP_PARAM))
            {
                xap = context.ApplicationInitParams[XAP_PARAM];
            }

            Debug.WriteLine("externalXap: {0}", xap);

            if (!string.IsNullOrEmpty(xap))
            {
                Download(xap);
            }
        }

        private void Download(string xap)
        {
            var wc = new WebClient();
            wc.OpenReadCompleted += WcOpenReadCompleted;
            wc.OpenReadAsync(new Uri(xap, UriKind.Relative));
        }

        void WcOpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                Debug.WriteLine(e.Error.Message);
            }
            else
            {
                ProcessResult(e.Result);
            }
        }

        private static void ProcessResult(Stream result)
        {
            var manifestStream = Application.GetResourceStream(
            new StreamResourceInfo(result, null),
            new Uri("AppManifest.xaml", UriKind.Relative));

            var appManifest = new StreamReader(manifestStream.Stream)
                .ReadToEnd();
            var reader = XmlReader.Create(new StringReader(appManifest));
            Assembly asm = null;
            while (reader.Read())
            {
                if (!reader.IsStartElement("AssemblyPart")) continue;

                reader.MoveToAttribute("Source");
                reader.ReadAttributeValue();

                if (!reader.Value.EndsWith(".dll") 
                    || !reader.Value.StartsWith("HelloSilverlight")) continue;

                var assemblyStream = new StreamResourceInfo(result, 
                    "application/binary");
                var si = Application.GetResourceStream(assemblyStream, 
                    new Uri(reader.Value, UriKind.Relative));
                var p = new AssemblyPart();
                asm = p.Load(si.Stream);
                break;
            }

            if (asm == null)
                return;

            var type = (from t in asm.GetTypes() 
                        where t.FullName.StartsWith("HelloSilverlight") 
                        select t).FirstOrDefault();

            Activator.CreateInstance(type);
        }

        public void StopService()
        {            
        }
    }
}