﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Windows;
using IsolatedStorageExample.Model;

namespace IsolatedStorageExample.Encryption
{
    public class Encrypting
    {
        private const string PASSWORD = "Another Super Secret";

        public void Encrypt(Note note, Stream stream)
        {
            using (var aes = new AesManaged())
            {
                var deriveBytes = 
                    new Rfc2898DeriveBytes(PASSWORD, 
                        Encoding.UTF8.GetBytes(
                        string.Format("xxxxxxxx{0}", 
                        note.Name)));
                aes.Key = deriveBytes.GetBytes(128 / 8);
                stream.Write(
                    BitConverter.GetBytes(aes.IV.Length), 
                    0, 
                    sizeof(int));
                stream.Write(aes.IV, 0, aes.IV.Length);

                using (var cs = new CryptoStream(
                    stream, 
                    aes.CreateEncryptor(), 
                    CryptoStreamMode.Write))
                {
                    var rawPlaintext = Encoding.Unicode.GetBytes(note.Contents);
                    cs.Write(rawPlaintext, 0, rawPlaintext.Length);
                    cs.FlushFinalBlock();
                }
            }
        }

        public void Decrypt(Note note, Stream stream)
        {
            using (Aes aes = new AesManaged())
            {
                var deriveBytes = new Rfc2898DeriveBytes(
                    PASSWORD,
                    Encoding.UTF8.GetBytes(
                    string.Format("xxxxxxxx{0}",note.Name)));
                aes.Key = deriveBytes.GetBytes(128 / 8);
                aes.IV = ReadByteArray(stream);
                var cs = new CryptoStream(stream, 
                    aes.CreateDecryptor(), 
                    CryptoStreamMode.Read);
                var reader = new StreamReader(cs, 
                    Encoding.Unicode);
                try
                {
                    note.Contents = reader.ReadToEnd();
                    reader.Dispose();
                    cs.Dispose();
                    return;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(
                        string.Format("Failed to decrypt: {0}", 
                        ex.Message));
                }
            }
        }

        private static byte[] ReadByteArray(Stream s)
        {
            var rawLength = new byte[sizeof(int)];
            if (s.Read(rawLength, 0, rawLength.Length) != rawLength.Length)
            {
                throw new SystemException("Stream did not contain properly formatted byte array");
            }

            var buffer = new byte[BitConverter.ToInt32(rawLength, 0)];
            if (s.Read(buffer, 0, buffer.Length) != buffer.Length)
            {
                throw new SystemException("Did not read byte array properly");
            }

            return buffer;
        }

    }
}