﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Security.Cryptography;
using System.Text;
using System.Windows;

namespace IsolatedStorageExample.Encryption
{
    public class Signing
    {
        private const string PASSWORD = "Super secret password";

        public void CreateSignature(string sourceFile, string destFile)
        {
            var deriveBytes = new Rfc2898DeriveBytes(PASSWORD,
                                                     Encoding.UTF8.GetBytes(sourceFile));
            var key = deriveBytes.GetBytes(64);
            using (var isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (var outStream = isoStore.OpenFile(destFile, FileMode.Create))
                {
                    try
                    {
                        var myhmacsha256 = new HMACSHA256(key);
                        var inStream = isoStore.OpenFile(sourceFile, FileMode.Open);
                        inStream.Position = 0;
                        var hashValue = myhmacsha256.ComputeHash(inStream);
                        inStream.Position = 0;
                        outStream.Write(hashValue, 0, hashValue.Length);
                        myhmacsha256.Clear();
                        inStream.Close();
                        outStream.Close();
                        return;
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format("Signature failed: {0}",
                                                      ex.Message));
                    }
                }
            }
        }

        public void VerifySignature(string sourceFile, string signFile)
        {
            var deriveBytes = new Rfc2898DeriveBytes(PASSWORD,
                                                        Encoding.UTF8.GetBytes(sourceFile));
            var key = deriveBytes.GetBytes(64);
            using (var isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                try
                {
                    var myhmacsha256 = new HMACSHA256(key);
                    var inStream = isoStore.OpenFile(sourceFile, FileMode.Open);
                    inStream.Position = 0;
                    var hashValue = myhmacsha256.ComputeHash(inStream);
                    myhmacsha256.Clear();
                    inStream.Close();

                    var savedBytes = new byte[hashValue.Length];
                    var hashStream = isoStore.OpenFile(signFile, FileMode.Open);
                    hashStream.Read(savedBytes, 0, hashValue.Length);
                    hashStream.Close();

                    var areSame = true;
                    for (var x = 0; x < hashValue.Length && areSame; x++)
                    {
                        areSame = hashValue[x] == savedBytes[x];
                    }
                    if (!areSame)
                    {
                        MessageBox.Show(string.Format(
                            "Warning! Note at {0} has been tampered with!",
                            sourceFile));
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(string.Format("Failed to verify signature: {0}",
                                                    ex.Message));
                }
            }
        }
    }
}