﻿using System;
using System.Collections.Generic;
using System.IO;

namespace LargeDataSets.Web
{
    public partial class GenerateData : System.Web.UI.Page
    {
        private readonly List<string> _lastNames = new List<string>();
        private readonly List<string> _firstNames = new List<string>();
        private readonly List<string> _addresses = new List<string>();
        private readonly List<string> _city = new List<string>();
        private readonly List<string> _states = new List<string>();
        private readonly List<string> _emails = new List<string>(); 

        protected void Page_Load(object sender, EventArgs e)
        {
            using (
                var stream =
                    typeof (GenerateData).Assembly.GetManifestResourceStream(
                        "LargeDataSets.Web.SampleData.randomdata.csv"))
            {
                if (stream == null)
                {
                    throw new InvalidDataException("Couldn't open stream.");
                }

                var first = true;

                using (var textReader = new StreamReader(stream))
                {
                    while (!textReader.EndOfStream)
                    {
                        var next = textReader.ReadLine();
                        if (string.IsNullOrEmpty(next))
                        {
                            break;
                        }

                        if (first)
                        {
                            first = false;
                            continue;
                        }

                        var parts = next.Split(',');
                        _lastNames.Add(parts[0]);
                        _firstNames.Add(parts[1]);
                        _addresses.Add(parts[2]);
                        _city.Add(parts[3]);
                        _states.Add(parts[4]);
                        _emails.Add(parts[5]);
                    }
                }

                var random = new Random();

                using (var context = new ContactsEntities())
                {
                    for (var x = 0; x < 1000000; x++)
                    {
                        var contact = new Contact
                                          {
                                              Address = _addresses[random.Next(_addresses.Count)],
                                              City = _city[random.Next(_city.Count)],
                                              Email = _emails[random.Next(_emails.Count)],
                                              FirstName = _firstNames[random.Next(_firstNames.Count)],
                                              LastName = _lastNames[random.Next(_lastNames.Count)],
                                              State = _states[random.Next(_states.Count)]
                                          };
                        context.Contacts.AddObject(contact);                       
                        if (x % 500 == 0)
                        {
                            context.SaveChanges();
                        }
                    }                    
                }
            }
        }
    }
}