﻿using System;
using System.Collections.Generic;
using System.ServiceModel.DomainServices.Client;
using LargeDataSets.Web;

namespace LargeDataSets
{
    public class Repository : IRepository
    {
        private int _totalCount;
        private readonly ContactContext _contactContext = new ContactContext();

        public Repository()
        {
            var query = (
                            from c
                                in _contactContext.GetContactsQuery()
                            select c).Take(1);
            query.IncludeTotalCount = true;
            _contactContext.Load(
                query,
                callback => { _totalCount = callback.TotalEntityCount; }, null);
        }

        public void ProcessPage(int page, int pageSize, 
            Action<IEnumerable<Contact>> callback)
        {
            var take = pageSize;
            var skip = pageSize*(page - 1);
            var query = (from c in _contactContext.GetContactsQuery()
                            select c).Skip(skip).Take(take);
            query.IncludeTotalCount = true;
            _contactContext.Load(
                query,
                cb =>
                    {
                        _totalCount = cb.TotalEntityCount;
                        callback(cb.Entities);
                    }, null);
        }

        public int GetTotalPages(int pageSize)
        {
            return (_totalCount/pageSize) + 1;
        }
    }
}