﻿using System.Collections.Generic;
using System.ComponentModel;
using Jounce.Core.Command;
using Jounce.Framework.Command;
using LargeDataSets.Web;

namespace LargeDataSets
{
    public class ViewModel : INotifyPropertyChanged
    {
        private List<Contact> _contacts = new List<Contact>();
        private readonly IRepository _repository;

        public int CurrentPage { get; set; }

        public int TotalPages { get; set; }

        public IActionCommand NextPage { get; set; }

        public IActionCommand PreviousPage { get; set; }

        public IEnumerable<Contact> Contacts
        {
            get
            {
                if (_contacts.Count == 0)
                {
                    Refresh();
                }

                return _contacts;
            }

            set { _contacts = new List<Contact>(value); }
        }

        public ViewModel()
        {
            Contacts = new List<Contact>();
            NextPage = new ActionCommand<object>(obj => GoToNextPage(),
                                                    obj => CurrentPage < TotalPages);
            PreviousPage = new ActionCommand<object>(obj => GoToPreviousPage(),
                                                    obj => CurrentPage > 1);

            if (!DesignerProperties.IsInDesignTool)
            {
                CurrentPage = 1;
                _repository = new Repository();                
            }
            else
            {
                CurrentPage = 2;
                TotalPages = 10;
            }
        }

        private void RaiseChanges()
        {
            RaisePropertyChange("Contacts");
            RaisePropertyChange("TotalPages");
            RaisePropertyChange("CurrentPage");
            NextPage.RaiseCanExecuteChanged();
            PreviousPage.RaiseCanExecuteChanged();
        }
        
        private void Refresh()
        {
            _repository.ProcessPage(
                CurrentPage,
                20,
                cb =>
                    {
                        TotalPages = _repository.GetTotalPages(20);
                        _contacts = new List<Contact>(cb);
                        RaiseChanges();
                    });
        }       

        private void GoToPreviousPage()
        {
            CurrentPage--;
            Refresh();
        }

        private void GoToNextPage()
        {
            CurrentPage++;
            Refresh();
        }
        
        protected void RaisePropertyChange(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}