﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;

namespace MEFIntro
{
    public partial class App
    {

        public App()
        {
            this.Startup += this.Application_Startup;
            this.Exit += this.Application_Exit;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        [Import("Shell")]
        public UserControl RootVisualControl { get; set; }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            var catalog = new AggregateCatalog(new DeploymentCatalog());

            var wc = new WebClient();
            wc.DownloadStringCompleted +=
                (o, args) =>
                    {
                        var list = args.Result.Split(
                            new[]{Environment.NewLine},
                            StringSplitOptions.RemoveEmptyEntries);
                        foreach (var deploymentCatalog 
                            in list.Select(extension => 
                                new DeploymentCatalog(extension)))
                        {
                            catalog.Catalogs.Add(deploymentCatalog);
                            deploymentCatalog.DownloadAsync();
                        }
                    };

            var uri = new Uri("../get.plugins", UriKind.Relative);
            wc.DownloadStringAsync(uri);

            var container = new CompositionContainer(catalog);
            CompositionHost.Initialize(container);

            for (var x = 0; x < 5; x++)
            {
                container.ComposeExportedValue("Item", x.ToString());    
            }

            CompositionInitializer.SatisfyImports(this);            
            this.RootVisual = RootVisualControl;

            var batch = new CompositionBatch();
            for (var x = 5; x < 10; x++)
            {
                batch.AddExportedValue("Item", x.ToString());                
            }
            container.Compose(batch);            
        }

        private void Application_Exit(object sender, EventArgs e)
        {

        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                Deployment.Current.Dispatcher.BeginInvoke(delegate { ReportErrorToDOM(e); });
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }
    }
}
