﻿using System.Collections.Generic;
using System.ComponentModel.Composition;
using Jounce.Core.Event;
using Jounce.Core.Model;
using Jounce.Core.View;
using Jounce.Core.ViewModel;
using Jounce.Framework;

namespace ManualNavigation.ViewModels
{
    [Export]
    public class BackStack : BaseNotify, IEventSink<GoBackMessage>, 
        IEventSink<ViewNavigationArgs>, IPartImportsSatisfiedNotification
    {
        private readonly Stack<string> _stack = new Stack<string>();

        public bool CanGoBack
        {
            get { return _stack.Count > 1; }
        }

        [Import]
        public IEventAggregator EventAggregator { get; set; }

        [Import]
        public IViewModelRouter Router { get; set; }

        public void HandleEvent(GoBackMessage publishedEvent)
        {
            if (!CanGoBack) return;

            _stack.Pop(); // current view
            EventAggregator.Publish(_stack.Pop().AsViewNavigationArgs());

            RaisePropertyChanged(()=>CanGoBack);
        }

        public void HandleEvent(ViewNavigationArgs publishedEvent)
        {
            var metadata = Router.GetViewMetadata(publishedEvent.ViewType);

            if (metadata == null || !metadata.Category.Equals("Menu"))
            {
                return;
            }

            _stack.Push(publishedEvent.ViewType);
            RaisePropertyChanged(()=>CanGoBack);
        }

        public void OnImportsSatisfied()
        {
            EventAggregator.Subscribe<GoBackMessage>(this);
            EventAggregator.Subscribe<ViewNavigationArgs>(this);
        }
    }
}