﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Controls;
using Jounce.Core.Command;
using Jounce.Core.Event;
using Jounce.Core.View;
using Jounce.Core.ViewModel;
using Jounce.Framework.Command;
using ManualNavigation.Views;

namespace ManualNavigation.ViewModels
{
    [ExportAsViewModel("Jounce")]
    public class JounceViewModel : BaseViewModel, IEventSink<ViewNavigationArgs>
    {
        private string _lastControl = string.Empty;
        private readonly Dictionary<string,UserControl> _controls 
            = new Dictionary<string, UserControl>(); 

        public JounceViewModel()
        {
            Views = new ObservableCollection<UserControl>();

            GoBackCommand = new ActionCommand<object>(
                obj => EventAggregator.Publish(new GoBackMessage()),
                obj => CanGoBack);

            if (InDesigner)
            {
                Views.Add(new Panel1());
            }
        }

        [Import]
        public BackStack Back { get; set; }

        public bool CanGoBack
        {
            get { return Back != null && Back.CanGoBack; }
        }

        public ObservableCollection<UserControl> Views { get; private set; }

        public IActionCommand GoBackCommand { get; private set; }

        public void HandleEvent(ViewNavigationArgs publishedEvent)
        {
            var viewMetadata = Router.GetMetadataForView(publishedEvent.ViewType);

            if (viewMetadata == null || !viewMetadata.Category.Equals("Menu"))
            {
                return;
            }

            if (!string.IsNullOrEmpty(_lastControl))
            {                
                var lastControl = _controls[_lastControl];
                VisualStateManager.GoToState(lastControl, "Hide", true);
            }

            _lastControl = publishedEvent.ViewType;

            UserControl control;
            
            if (_controls.ContainsKey(_lastControl))
            {
                control = _controls[_lastControl];
            }
            else
            {
                control = Router.GetNonSharedView(_lastControl, null);
                Views.Add(control);
                _controls.Add(_lastControl, control);
            }
            
            VisualStateManager.GoToState(control, "Show", true);
        }

        protected override void InitializeVm()
        {
            EventAggregator.Subscribe(this);
            Back.PropertyChanged += (o, e) => GoBackCommand.RaiseCanExecuteChanged();
        }
    }
}