﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Windows.Controls;
using System.Linq;
using Jounce.Core.Command;
using Jounce.Core.Event;
using Jounce.Core.View;
using Jounce.Core.ViewModel;
using Jounce.Framework;
using Jounce.Framework.Command;

namespace ManualNavigation.ViewModels
{
    [ExportAsViewModel("Navigation")]
    public class NavigationViewModel : BaseViewModel, IEventSink<ViewNavigationArgs>
    {
        private string _lastView = string.Empty;

        [ImportMany]
        public Lazy<UserControl, IExportAsViewMetadata>[] Views { get; set; }

        public NavigationViewModel()
        {
            NavigateCommand = new ActionCommand<string>(
                s => EventAggregator.Publish(s.AsViewNavigationArgs()),
                s => !_lastView.Equals(s));

            NavigationItems = new ObservableCollection<IExportAsViewMetadata>();

            if (!InDesigner) return;

            var sampleMenuItem = new ExportAsViewAttribute("Sample")
                                     {
                                         MenuName = "Example Menu", 
                                         ToolTip = "Example Tooltip"
                                     };
            NavigationItems.Add(sampleMenuItem);
        }

        public IActionCommand<string> NavigateCommand { get; private set; }

        public ObservableCollection<IExportAsViewMetadata> NavigationItems { get; private set; }

        protected override void InitializeVm()
        {
            var views = (from v
                                in Views
                            where v.Metadata.Category.Equals("Menu")
                            select v.Metadata);
            foreach(var viewMetadata in views)
            {
                NavigationItems.Add(viewMetadata);
            }

            EventAggregator.Subscribe(this);
        }

        public void HandleEvent(ViewNavigationArgs publishedEvent)
        {
            var metadata = Router.GetViewMetadata(publishedEvent.ViewType);

            if (metadata != null && metadata.Category.Equals("Menu"))
            {
                _lastView = publishedEvent.ViewType;
                NavigateCommand.RaiseCanExecuteChanged();
            }
        }
    }
}