﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;

namespace ManualNavigation.Views
{
    public partial class ItemsControlNavigation
    {
        private readonly Dictionary<string, UserControl> _controls;

        private UserControl _lastControl;
        private readonly List<UserControl> _userControls = new List<UserControl>();

        public ItemsControlNavigation()
        {
            _controls = new Dictionary<string, UserControl>
                            {
                                {"Text", ViewHelper.MakeUserControl(ViewHelper.MakeTextBlock())},
                                {"Circle", ViewHelper.MakeUserControl(ViewHelper.MakeEllipse())},
                                {"Square", ViewHelper.MakeUserControl(ViewHelper.MakeSquare())}
                            };

            _userControls.AddRange(_controls.Values);            

            InitializeComponent();

            MainRegion.ItemsSource = _userControls;
        }

        private void ButtonClick(object sender, RoutedEventArgs e)
        {
            var button = sender as Button;

            if (button == null)
            {
                return;
            }

            if (_lastControl != null)
            {
                _lastControl.Visibility = Visibility.Collapsed;
            }

            var key = button.Content.ToString();

            if (!_controls.ContainsKey(key)) return;

            var control = _controls[key];
            control.Visibility = Visibility.Visible;            
            _lastControl = control;
        }        
    }
}
