﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace MetadataExample
{
    public partial class App : Application
    {

        public App()
        {
            this.Startup += this.Application_Startup;
            this.Exit += this.Application_Exit;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            var container = new CompositionContainer(
                new DeploymentCatalog());
            CompositionHost.Initialize(container);            
            this.RootVisual = new MainPage();
            ParseContainer(container);
        }

        private void ParseContainer(CompositionContainer container)
        {
            var catalog = container.Catalog;
            foreach(var part in catalog.Parts)
            {
                Debug.WriteLine("Part.{0}===========",
                    part);                
                if (part.Metadata.Keys.Count > 0)
                {
                    foreach(var key in part.Metadata.Keys)
                    {
                        Debug.WriteLine("Part.Metadata.{0}={1}",
                            key,
                            part.Metadata[key]);
                    }
                }
                foreach(var import in part.ImportDefinitions)
                {
                    Debug.WriteLine("Part.Imports.{0}.Constraint={1}", 
                        import.ContractName,
                        import.Constraint);
                }
                foreach(var export in part.ExportDefinitions)
                {
                    Debug.WriteLine("Part.Exports.{0}", export.ContractName);
                    if (export.Metadata.Keys.Count > 0)
                    {
                        foreach(var key in export.Metadata.Keys)
                        {
                            Debug.WriteLine("Part.Exports.{0}.Metadata.{1}={2}",
                                export.ContractName,
                                key,
                                export.Metadata[key]);
                        }
                    }
                }
            }
        }

        private void Application_Exit(object sender, EventArgs e)
        {

        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                Deployment.Current.Dispatcher.BeginInvoke(delegate { ReportErrorToDOM(e); });
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }
    }
}
