﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace MetadataExample
{
    public partial class MainPage
    {
        private const string ENGLISH = "English";
        private const string SPANISH = "Spanish";

        private bool _isSpanish; 

        [ImportMany]
        public Lazy<string, Dictionary<string,object>>[]
            WelcomeMessages { get; set; }

        [ImportMany]
        public Lazy<UserControl, IControlTypeMetadata>[]
            Shapes { get; set; }

        public MainPage()
        {
            InitializeComponent();

            if (DesignerProperties.IsInDesignTool)
            {
                WelcomeText.Text = "Designer Text";
                return;
            }

            CompositionInitializer.SatisfyImports(this);
            ShowLanguage();
            LoadShapes();
        }

        private void LoadShapes()
        {
            foreach(var shapeDefinition in Shapes)
            {
                Debug.WriteLine("Inspecting item...");
                var metadata = shapeDefinition.Metadata;
                if (metadata.ControlType == ControlTypes.Ellipse)
                {
                    Ellipses.Items.Add(shapeDefinition.Value);
                }
                else
                {
                    Rectangles.Items.Add(shapeDefinition.Value);
                }
            }
        }

        private void ShowLanguage()
        {
            var language = _isSpanish ? SPANISH : ENGLISH;

            var match = (WelcomeMessages
                .Where(item =>
                       item.Metadata.ContainsKey("Language") &&
                       item.Metadata["Language"].Equals(language))
                       .Select(item => item))
                       .FirstOrDefault();

            WelcomeText.Text = match == null ? "Language Not Found" : match.Value;
        }

        private void LanguageButton_Click(object sender, RoutedEventArgs e)
        {
            _isSpanish = !_isSpanish;
            LanguageButton.Content = _isSpanish ? ENGLISH : SPANISH;
            ShowLanguage();
        }
    }
}
