﻿using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TextExamples
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }

        private void RichTextBox_Loaded(object sender, RoutedEventArgs e)
        {
            var rtb = sender as RichTextBox;
            if (rtb == null)
            {
                return;
            }
            foreach (var block in rtb.Blocks)
            {
                Debug.WriteLine(block);
                rtb.Selection.Select(block.ContentStart, block.ContentEnd);
                Debug.WriteLine(rtb.Selection.Text);
            }
        }

        private static char GetCurrentChar(RichTextBox rtb, TextPointer pointer)
        {
            var nextPointer = pointer.GetNextInsertionPosition(LogicalDirection.Forward);

            if (nextPointer != null)
            {
                rtb.Selection.Select(pointer, nextPointer);

                if (rtb.Selection.Text.Length != 0)
                    return rtb.Selection.Text[0];
            }

            return (char) 0x0;
        }


        private void Button_Click(object sender, RoutedEventArgs e)
        {
            SelectedText.Text = RichText.Selection.Text;
            RichText.Selection.ApplyPropertyValue(TextElement.FontWeightProperty, FontWeights.Bold);
            
            var start = RichText.ContentStart
                .GetNextInsertionPosition(LogicalDirection.Forward);

            var end = start.GetNextInsertionPosition(LogicalDirection.Forward);

            while (!char.IsWhiteSpace(GetCurrentChar(RichText, end)))
            {
                end = end.GetNextInsertionPosition(LogicalDirection.Forward);
            }

            var rect = RichText.ContentStart
                .GetCharacterRect(LogicalDirection.Forward);
            rect.Union(end.GetCharacterRect(LogicalDirection.Forward));

            var rectangle =
                new Rectangle
                    {
                        Fill = new SolidColorBrush(Colors.Yellow),
                        Opacity = 0.3,
                        Width = rect.Width,
                        Height = rect.Height,
                        Margin = new Thickness(5),
                        HorizontalAlignment = HorizontalAlignment.Left,
                        VerticalAlignment = VerticalAlignment.Top,
                    };

            rectangle.SetValue(Grid.ColumnProperty, 1);

            ((Grid) VisualTreeHelper
                        .GetParent(RichText))
                .Children.Add(rectangle);       
        }
    }
}