﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ToDoList.CLRTests.Mocks;
using ToDoList.Model;

namespace ToDoList.CLRTests
{
    /// <summary>
    /// Summary description for ToDoItemTests
    /// </summary>
    [TestClass]
    public class ToDoItemTests
    {
        public ToDoItemTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        private ToDoItem _target;
        private MockPublisherSubscriber _mockPublisherSubscriber;

        [TestInitialize]
        public void Initialize()
        {
            _target = new ToDoItem();
            _mockPublisherSubscriber = new MockPublisherSubscriber();
            _target.PubSub = _mockPublisherSubscriber;
        }       

        [TestMethod]
        public void WhenMarkedCompleteExpectIsInFutureIsAlwaysFalse()
        {
            _target.DueDate = DateTime.Now.AddDays(1);
            _target.MarkComplete();
            Assert.IsFalse(_target.IsInFuture, 
                "Is in future should be false when marked complete.");
        }

        [TestMethod]
        public void WhenMarkedCompleteExpectIsPastDueIsAlwaysFalse()
        {
            _target.DueDate = DateTime.Now.AddDays(-1);
            _target.MarkComplete();
            Assert.IsFalse(_target.IsPastDue,
                "Is past due should be false when marked complete.");
        }

        [TestMethod]
        public void WhenMarkedCompleteExpectIsDueTomorrowIsAlwaysFalse()
        {
            _target.DueDate = DateTime.Now.Date.AddDays(1);
            _target.MarkComplete();
            Assert.IsFalse(_target.IsDueTomorrow,
                "Is due tomorrow should be false when marked complete.");
        }

        [TestMethod]
        public void WhenMarkedCompleteExpectIsDueNextWeekIsAlwaysFalse()
        {
            _target.DueDate = DateTime.Now.AddDays(6);
            if (_target.DueDate.DayOfWeek.Equals(DayOfWeek.Sunday))
            {
                _target.DueDate = _target.DueDate.AddDays(1);
            }
            _target.MarkComplete();
            Assert.IsFalse(_target.IsDueNextWeek,
                "Is due next week should be false when marked complete.");
        }    
    
        [TestMethod]
        public void WhenMarkedCompleteExpectToDoItemCompleteMessage()
        {
            _target.MarkComplete();
            Assert.IsNotNull(_mockPublisherSubscriber.LastMessage,
                "Message should be published when marked complete.");
            Assert.IsInstanceOfType(_mockPublisherSubscriber.LastMessage,
                typeof(MessageToDoItemComplete),
                "Message type is incorrect.");
            Assert.AreEqual(_target,
                ((MessageToDoItemComplete)
                _mockPublisherSubscriber.LastMessage).Item,
                "Item in message should match the target.");
        }
    }
}
