﻿using System;
using System.Json;
using System.Net;
using System.Text;

namespace ToDoList.RESTRepository
{
    public delegate void ResourceCallback(JsonValue value, Exception ex);

    public static class JsonHelper
    {
        public static void GetResource(Uri resource, ResourceCallback callback)
        {
            var wc = new WebClient();
            wc.DownloadStringCompleted += DownloadStringCompleted;
            wc.DownloadStringAsync(resource, callback);
        }

        public static void PostResource(Uri resource, string data, ResourceCallback callback)
        {
            var wc = new WebClient();
           
            wc.UploadStringCompleted += WcUploadStringCompleted;
            wc.Headers["Content-type"] = "application/json";
            wc.Encoding = Encoding.UTF8;
            wc.UploadStringAsync(resource, "POST", data, callback);
        }

        static void WcUploadStringCompleted(object sender, UploadStringCompletedEventArgs e)
        {
            var wc = sender as WebClient;
            if (wc == null)
            {
                return;
            }
            wc.UploadStringCompleted -= WcUploadStringCompleted;
            var callback = e.UserState as ResourceCallback;
            if (callback == null)
            {
                return;
            }
            try
            {
                if (e.Error != null)
                {
                    throw e.Error;
                }
                callback(JsonValue.Parse(e.Result), null);
            }
            catch (Exception ex)
            {
                callback(null, ex);
            }
        }

        private static void DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            var wc = sender as WebClient; 
            if (wc == null)
            {
                return;
            }            
            wc.DownloadStringCompleted -= DownloadStringCompleted;
            var callback = e.UserState as ResourceCallback;
            if (callback == null)
            {
                return;
            }
            try
            {
                if (e.Error != null)
                {
                    throw e.Error;
                }
                callback(JsonValue.Parse(e.Result), null);
            }
            catch(Exception ex)
            {
                callback(null, ex);
            }
        }        

        public static JsonObject AsObject(this JsonValue value)
        {
            return value as JsonObject;
        }

        public static JsonArray AsArray(this JsonValue value)
        {
            return value as JsonArray;
        }
    }
}