﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Json;
using System.Linq;
using System.Windows;
using ToDoList.Contracts;
using ToDoList.Model;
using ToDoList.RESTRepository.Services;

namespace ToDoList.RESTRepository
{
    [Export(typeof (IRepository))]
    public class SLRestRepository : IRepository
    {
        private readonly object _mutex = new Object();
        private ToDoListServiceClient _client = new ToDoListServiceClient();

        private readonly List<IToDoItem> _items = new List<IToDoItem>();

        public SLRestRepository()
        {
            LoadTasks();
            _client.DeleteCompleted += ClientDeleteCompleted;
        }

        void ClientDeleteCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {            
            if (e.Error == null)
            {
                LoadTasks();
                return;
            }

            var dispatcher = Deployment.Current.Dispatcher;
            Action action = () => MessageBox.Show(e.Error.Message);
            if (dispatcher.CheckAccess())
            {
                action();
            }
            else
            {
                dispatcher.BeginInvoke(action);
            }
        }

        private void LoadTasks()
        {            
            JsonHelper.GetResource(new Uri("../REST/Tasks/", UriKind.Relative), EndLoadTasks);
        }

        [Import]
        public ExportFactory<ToDoItemOverride> ItemOverrideFactory { get; set; }

        private void EndLoadTasks(JsonValue value, Exception ex)
        {
            if (ex != null)
            {
                throw ex;
            }
            lock (_mutex)
            {
                _items.Clear();
                var list = value.AsObject()["Item"].AsArray();
                foreach (var todo in list.Select(
                    item =>
                        {
                            var task = item["Item"].AsObject();
                            var taskItem = ItemOverrideFactory.CreateExport().Value;
                            taskItem.CompletedDate = task["CompletedDate"];
                            taskItem.Description = task["Description"];
                            taskItem.DueDate = task["DueDate"];
                            taskItem.Id = task["Id"];
                            taskItem.IsComplete = task["IsComplete"];
                            taskItem.Link = task["Link"];
                            taskItem.Title = task["Title"];
                            return taskItem;                        
                        }))
                {
                    _items.Add(todo);
                }
            }
            RaiseAggregateRootChanged();
        }

        public IQueryable<IToDoItem> Query()
        {
            var retVal = new List<IToDoItem>();
            lock (_mutex)
            {
                retVal.AddRange(_items);
            }
            return _items.AsQueryable();
        }

        public void Save(IToDoItem item)
        {
            var properties = new Dictionary<string, JsonValue>
                                    {
                                        {"CompletedDate", item.CompletedDate},
                                        {"Description", item.Description},
                                        {"DueDate", item.DueDate},
                                        {"Id", item.Id},
                                        {"IsComplete", item.IsComplete},
                                        {"Link", item.Link},
                                        {"Title", item.Title}
                                    };
            var json = new JsonObject(properties);
            JsonHelper.PostResource(
                new Uri(string.Format("../REST/Task/{0}", item.Id), UriKind.Relative),
                json.ToString(),
                (o, e) => LoadTasks());
        }
        
        public void Delete(IToDoItem item)
        {
            _client.DeleteAsync(item.Id);            
        }        

        public event EventHandler AggregateRootChanged;

        protected void RaiseAggregateRootChanged()
        {
            var handler = AggregateRootChanged;
            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
        }               
    }
}