﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using ToDoList.Contracts;

namespace ToDoList.Web
{
    public class ToDoSocketServer 
    {
        private TcpListener _listener;
        private readonly ManualResetEvent _clientConnected = 
            new ManualResetEvent(false);
        private readonly List<StreamWriter> _streams = new List<StreamWriter>();
            
        [Import]
        public IRepository Repository { get; set; }

        public void Start()
        {
            _listener = new TcpListener(IPAddress.Any, 4510);
            _listener.Start();
            Repository.AggregateRootChanged += RepositoryAggregateRootChanged;
            while(true)
            {
                try
                {
                    _clientConnected.Reset();
                    _listener.BeginAcceptTcpClient(
                        OnClientConnected, null);
                    _clientConnected.WaitOne();
                }
                catch(Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
            }
        }

        void RepositoryAggregateRootChanged(object sender, EventArgs e)
        {
            var issues = new List<StreamWriter>();
            foreach (var stream in _streams
                .Where(stream => stream != null))
            {
                try
                {
                    stream.WriteLine("Refresh");
                }
                catch
                {
                    issues.Add(stream);
                }
            }
            foreach(var issue in issues)
            {
                _streams.Remove(issue);
            }
        }

        private void OnClientConnected(IAsyncResult ar)
        {
            _clientConnected.Set();
            var client = _listener.EndAcceptTcpClient(ar);
            if (!client.Connected) return;
            var stream = new StreamWriter(client.GetStream())
                                {
                                    AutoFlush = true
                                };
            _streams.Add(stream);
        }
    }
}