﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using Jounce.Framework;

namespace ToDoList.Repository
{
    public class ClientListener
    {
        private Action _callback;

        public void Start(Action callback)
        {
            _callback = callback;
            var endPoint = new DnsEndPoint(
                Application.Current.Host.Source.DnsSafeHost, 
                4510, 
                AddressFamily.InterNetwork);
            var socket = new Socket(
                AddressFamily.InterNetwork, 
                SocketType.Stream, 
                ProtocolType.Tcp);
            var args = new SocketAsyncEventArgs
                            {
                                UserToken = socket, 
                                RemoteEndPoint = endPoint,
                                SocketClientAccessPolicyProtocol 
                                = SocketClientAccessPolicyProtocol.Http
                            };
            args.Completed += OnSocketConnectCompleted;
            socket.ConnectAsync(args);  
        }

        private void OnSocketConnectCompleted(object sender, SocketAsyncEventArgs e)        
        {
            if (e.SocketError == SocketError.Success)
            {
                var response = new byte[16];
                e.SetBuffer(response, 0, response.Length);
                e.Completed -= OnSocketConnectCompleted;
                e.Completed += OnSocketReceive;
                var socket = (Socket) e.UserToken;
                socket.ReceiveAsync(e);
            }
            else
            {
                JounceHelper.ExecuteOnUI(
                    () => 
                        MessageBox.Show(e.SocketError.ToString()));
            }
        }

        private void OnSocketReceive(object sender, SocketAsyncEventArgs e)
        {
            try
            {
                var data = Encoding.UTF8.GetString(
                    e.Buffer, 
                    e.Offset, 
                    e.BytesTransferred);

                if (data.Contains("Refresh"))
                {
                    _callback();
                }
            }
            finally
            {
                var socket = (Socket) e.UserToken;
                socket.ReceiveAsync(e);
            }
        }    
    }
}