﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using ToDoList.Contracts;
using ToDoList.Model;
using ToDoList.Web;

namespace ToDoList.Repository
{
    [Export(typeof(IRepository))]
    public class WcfRiaRepository : IRepository 
    {        
        readonly ToDoRiaContext _context = new ToDoRiaContext();
        private readonly EntitySet<ToDoItemRia> _tasks;

        [Import]
        public ExportFactory<ToDoItemOverride> ToDoItemFactory { get; set; }

        public WcfRiaRepository()
        {
            _tasks = _context.ToDoItemRias;          
            LoadTasks();  
        }

        private void LoadTasks()
        {
            var query = _context.GetTasksQuery();
            _context.Load(query, LoadCallback, null);
        }

        private void LoadCallback(LoadOperation<ToDoItemRia> obj)
        {
            RaiseAggregateRootChanged();
        }

        public IQueryable<IToDoItem> Query()
        {
            return _tasks.Select(FromRia).AsQueryable();            
        }

        public void Save(IToDoItem item)
        {
            var existing = GetById(item.Id);
            if (existing != null)
            {
                FromTask(existing, item);                
            }
            else
            {
                _tasks.Add(FromTask(item));
            }
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(),
                null);
        }

        public void Delete(IToDoItem item)
        {
            var riaItem = GetById(item.Id);
            if (riaItem == null) return;
            _context.ToDoItemRias.Remove(riaItem);
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(), 
                null);
        }

        private ToDoItemRia GetById(Guid id)
        {
            return (from t in _tasks
                    where t.Id.Equals(id)
                    select t).FirstOrDefault();
        }

        private IToDoItem FromRia(ToDoItemRia task)
        {
            var newTask = ToDoItemFactory.CreateExport().Value;
            newTask.CompletedDate = task.CompletedDate;
            newTask.Description = task.Description;
            newTask.DueDate = task.DueDate;
            newTask.Id = task.Id;
            newTask.IsComplete = task.IsComplete;
            newTask.Link = task.Link;
            newTask.Title = task.Title;
            return newTask;
        }

        private static void FromTask(ToDoItemRia riaItem, IToDoItem task)
        {
            riaItem.CompletedDate = task.CompletedDate;
            riaItem.Description = task.Description;
            riaItem.DueDate = task.DueDate;
            riaItem.IsComplete = task.IsComplete;
            riaItem.Link = task.Link;
            riaItem.Title = task.Title;
        }

        private static ToDoItemRia FromTask(IToDoItem task)
        {
            var newTask = new ToDoItemRia {Id = Guid.NewGuid()};
            FromTask(newTask, task);
            return newTask;
        }

        protected void RaiseAggregateRootChanged()
        {
            var handler = AggregateRootChanged;
            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
        }

        public event EventHandler AggregateRootChanged;
    }
}