﻿using System;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using ToDoList.Contracts;
using ToDoList.Model;
using ToDoList.Web;

namespace ToDoList.Repository
{
    [Export(typeof(IRepository))]
    public class WcfRiaRepository : IRepository 
    {        
        readonly ToDoRiaContext _context = new ToDoRiaContext();
        private readonly EntitySet<ToDoItemRia> _tasks;
        private bool _ignoreSocket;
        private readonly ClientListener _listener = new ClientListener();

        private readonly Mapper<ToDoItemRia, ToDoItem> _mapper =
            new Mapper<ToDoItemRia, ToDoItem>(
                new[] {"Id", "CompletedDate", "IsComplete"});
            
        [Import]
        public ExportFactory<ToDoItemOverride> ToDoItemFactory { get; set; }

        public WcfRiaRepository()
        {
            _tasks = _context.ToDoItemRias;
            LoadTasks();
            _listener.Start(ServerUpdated);
        }

        private void ServerUpdated()
        {
            if (!_ignoreSocket)
            {
                LoadTasks();                
            }
        }

        private void LoadTasks()
        {
            _ignoreSocket = true;
            var query = _context.GetTasksQuery();
            _context.Load(query, LoadCallback, null);
        }

        private void LoadCallback(LoadOperation<ToDoItemRia> obj)
        {
            RaiseAggregateRootChanged();
        }

        public IQueryable<IToDoItem> Query()
        {
            return _tasks.Select(FromRia).AsQueryable();            
        }

        public void Save(IToDoItem item)
        {
            _ignoreSocket = true;
            var existing = GetById(item.Id);
            if (existing != null)
            {
                FromTask(existing, item);                
            }
            else
            {
                _tasks.Add(FromTask(item));
            }
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(),
                null);
        }

        public void Delete(IToDoItem item)
        {
            _ignoreSocket = true;
            var riaItem = GetById(item.Id);
            if (riaItem == null) return;
            _context.ToDoItemRias.Remove(riaItem);
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(), 
                null);
        }

        private ToDoItemRia GetById(Guid id)
        {
            return (from t in _tasks
                    where t.Id.Equals(id)
                    select t).FirstOrDefault();
        }

        private IToDoItem FromRia(ToDoItemRia task)
        {
            var newTask = ToDoItemFactory.CreateExport().Value;
            _mapper.MapSourceToTarget(task, newTask);
            newTask.Id = task.Id;
            newTask.IsComplete = task.IsComplete;
            newTask.CompletedDate = task.CompletedDate;
            return newTask;
        }

        private void FromTask(ToDoItemRia riaItem, IToDoItem task)
        {
            _mapper.MapTargetToSource((ToDoItem)task, riaItem);            
        }

        private ToDoItemRia FromTask(IToDoItem task)
        {
            var newTask = new ToDoItemRia {Id = Guid.NewGuid()};
            FromTask(newTask, task);
            return newTask;
        }

        protected void RaiseAggregateRootChanged()
        {
            var handler = AggregateRootChanged;
            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
            _ignoreSocket = false;
        }

        public event EventHandler AggregateRootChanged;
    }
}