﻿using System.ComponentModel.Composition;
using System.IO.IsolatedStorage;
using System.Windows.Browser;
using ToDoList.Contracts;

namespace ToDoList
{
    [Export(typeof(IToDoListApplicationContext))]
    public class ToDoListApplicationContext
        : IToDoListApplicationContext
    {
        private readonly IsolatedStorageSettings _appSettings =
            IsolatedStorageSettings.ApplicationSettings;

        public ToDoListApplicationContext()
        {
            if (!HtmlPage.IsEnabled) return;

            HtmlPage.RegisterScriptableObject(GetType().Name, this);

            var pluginName = HtmlPage.Plugin.Id;
            HtmlPage.Window.Eval(string.Format(
                @"window.onbeforeunload = function () {{
                    var slApp = document.getElementById('{0}');
                    var result = slApp.Content.{1}.OnBeforeUnload();
                    if(result.length > 0)
                        return result;
                }}",
                pluginName, GetType().Name)
                );
        }

        [ScriptableMember]
        public string OnBeforeUnload()
        {
            return PromptBeforeClose 
                ? "You may have unsaved changes!" 
                : string.Empty;
        }


        private string _title;

        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                if (HtmlPage.IsEnabled)
                {
                    HtmlPage.Document.SetProperty("title", value);
                }
            }
        }

        public bool PromptBeforeClose { get; set; }

        const string FILTER_KEY = "Filter";
                
        public string FilterName
        {
            get
            {
                return _appSettings.Contains(FILTER_KEY)
                            ? (string) _appSettings[FILTER_KEY]
                            : string.Empty;
            }
            set { _appSettings[FILTER_KEY] = value; }
        }

        const string SORT_KEY = "Sort";

        public string SortName
        {
            get
            {
                return _appSettings.Contains(SORT_KEY)
                           ? (string) _appSettings[SORT_KEY]
                           : string.Empty;
            }
            set { _appSettings[SORT_KEY] = value; }
        }
    }
}