﻿using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using Jounce.Core.Event;
using Jounce.Core.View;
using Jounce.Core.ViewModel;
using Jounce.Framework;
using Microsoft.Expression.Interactivity.Core;
using ToDoList.Contracts;

namespace ToDoList.ViewModels
{
    [ExportAsViewModel(typeof(MainViewModel))]
    public class MainViewModel : BaseViewModel, IEventSink<UnhandledExceptionEvent>
    {
        [Import]
        public Global Globals { get; set; }

        [Import]
        public IToDoListApplicationContext AppContext { get; set; }

        public ICommand NewCommand { get; private set; }
        
        protected override void InitializeVm()
        {
            NewCommand = new ActionCommand(obj=>MakeNew());
            EventAggregator.SubscribeOnDispatcher(this);
            EventAggregator.Publish(Globals.ToDoListView.AsViewNavigationArgs());              
            EventAggregator.Publish(Globals.QuickAddView.AsViewNavigationArgs());
            EventAggregator.Publish(Globals.FilterView.AsViewNavigationArgs());
            EventAggregator.Publish(Globals.TextFilterView.AsViewNavigationArgs());
            EventAggregator.Publish(Globals.SortView.AsViewNavigationArgs()); 
            EventAggregator.Publish(Globals.EditView.AsViewNavigationArgs());
            EventAggregator.Publish(new ViewNavigationArgs(Globals.EditView)
                .AddNamedParameter("State", "Closed"));
            base.InitializeVm();
            AppContext.Title = Global.Constants.MAIN_TITLE;
        }

        private void MakeNew()
        {
            EventAggregator.Publish(new ViewNavigationArgs(Globals.EditView)
                .AddNamedParameter("State", "New"));            
        }

        public void HandleEvent(UnhandledExceptionEvent publishedEvent)
        {
            publishedEvent.Handled = true;
            var message = Debugger.IsAttached
                              ? publishedEvent.UncaughtException.ToString()
                              : publishedEvent.UncaughtException.Message;
            MessageBox.Show(message, "Unexpected Error", MessageBoxButton.OK);
        }      
    }
}