﻿using System;
using System.Windows.Input;
using Jounce.Core.ViewModel;
using Jounce.Framework.ViewModel;
using ToDoList.Contracts;
using ToDoList.Model;

namespace ToDoList.ViewModels
{
    [ExportAsViewModel(typeof(QuickAddViewModel))]
    public class QuickAddViewModel 
        : BaseEntityViewModel, IQuickAddViewModel 
    {
        private DateTime _dueDate;
        public DateTime DueDate
        {            
            get { return _dueDate; }
            set
            {                
                _dueDate = value;
                ClearErrors(() => DueDate);
                string error;
                if (!new ToDoItem().ValidateDueDate(value, out error))
                {
                    SetError(() => DueDate, error);
                }
                RaisePropertyChanged(() => DueDate);
            }
        }

        private string _title;
        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                ClearErrors(() => Title);
                string error;
                if (!new ToDoItem().ValidateTitle(value, out error))
                {
                    SetError(() => Title, error);
                }
                RaisePropertyChanged(() => Title);
            }
        }              
        
        ICommand IQuickAddViewModel.CommitCommand
        {
            get { return CommitCommand; }
        }

        protected override void OnCommitted()
        {
            var item = new ToDoItem
                            {
                                Title = Title, 
                                DueDate = DueDate
                            };
            EventAggregator.Publish(MessageNewToDoItem.Create(item));
        }

        protected override void ValidateAll()
        {
            ClearErrors(() => DueDate);
            ClearErrors(() => Title);
            var todo = new ToDoItem();
            string error;
            if (!todo.ValidateDueDate(DueDate, out error))
            {
                SetError(() => DueDate, error);
            }
            if (!todo.ValidateTitle(Title, out error))
            {
                SetError(() => Title, error);
            }
        }
    }
}