﻿using System;
using System.ComponentModel.Composition;
using System.Windows.Input;
using Jounce.Core.Application;
using Jounce.Core.ViewModel;
using Jounce.Framework.ViewModel;
using ToDoList.Contracts;
using ToDoList.Model;

namespace ToDoList.ViewModels
{
    [ExportAsViewModel(typeof(QuickAddViewModel))]
    public class QuickAddViewModel 
        : BaseEntityViewModel, IQuickAddViewModel
    {
        private IToDoItem _referenceItem;

        public QuickAddViewModel()
        {
            GetToDoItem = () => new ToDoItem();
            DueDate = DateTime.Now.Date.AddDays(1).ToString();
            _referenceItem = GetToDoItem();
        }
        
        [Import(typeof(Func<IToDoItem>),AllowDefault = true,AllowRecomposition = true)]
        public Func<IToDoItem> GetToDoItem { get; set; }
        
        private DateTime _dueDate;
        private string _dueDateDisplay;
        
        public string DueDate
        {
            get { return _dueDateDisplay; }
            set
            {
                ClearErrors(() => DueDate);
                _dueDateDisplay = value;
                    
                if (DateTime.TryParse(value, out _dueDate))
                {
                    string error;
                    if (_referenceItem != null && 
                        !_referenceItem.ValidateDueDate(_dueDate, out error))
                    {
                        SetError(() => DueDate, error);
                    }
                    RaisePropertyChanged(() => DueDate);
                }
                else
                {
                   SetError(() => DueDate, "Date must be in proper format.");
                }
            }
        }

        private string _title;
        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                ClearErrors(() => Title);
                string error;
                if (_referenceItem != null &&
                    !_referenceItem.ValidateTitle(value, out error))
                {
                    SetError(() => Title, error);
                }
                RaisePropertyChanged(() => Title);
            }
        }              
        
        ICommand IQuickAddViewModel.CommitCommand
        {
            get { return CommitCommand; }
        }

        protected override void OnCommitted()
        {
            _referenceItem.Title = Title;
            _referenceItem.DueDate = _dueDate;
            _referenceItem.CreateNew();
            Reset();            
        }

        protected override void ValidateAll()
        {
            ClearErrors(() => DueDate);
            ClearErrors(() => Title);            
            string error;
            if (!_referenceItem.ValidateDueDate(_dueDate, out error))
            {
                SetError(() => DueDate, error);
            }
            if (!_referenceItem.ValidateTitle(Title, out error))
            {
                SetError(() => Title, error);
            }
        }
        
        private void Reset()
        {
            _title = string.Empty;
            DueDate = DateTime.Now.Date.AddDays(1).ToString();
            _referenceItem = GetToDoItem();
            RaisePropertyChanged(() => Title);
            RaisePropertyChanged(() => DueDate);
        }

        protected override void InitializeVm()
        {
            _referenceItem = GetToDoItem();
        }
    }
}