﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using Jounce.Core.Event;
using Jounce.Core.ViewModel;
using ToDoList.Contracts;
using ToDoList.Filters;
using ToDoList.Model;

namespace ToDoList.ViewModels
{
    [ExportAsViewModel(typeof (ToDoListViewModel))]
    public class ToDoListViewModel : BaseViewModel, IToDoListViewModel, IEventSink<FilterBase>,
                                     IEventSink<MessageNewToDoItem>
    {
        private readonly ObservableCollection<IToDoItem> _tasks =
            new ObservableCollection<IToDoItem>();

        private FilterBase _filter = FilterBase.Create("Default", t => true);

        [Import]
        public IRepository Repository { get; set; }

        [Import]
        public Global Globals { get; set; }

        public IEnumerable<IToDoItem> Tasks
        {
            get
            {
                return from t in _tasks
                       where _filter.Filter(t)
                       select t;
            }
        }        

        public void HandleEvent(FilterBase publishedEvent)
        {
            _filter = publishedEvent;
            RaisePropertyChanged(() => Tasks);
        }

        protected override void ActivateView(string viewName, IDictionary<string, object> viewParameters)
        {
            _tasks.Clear();

            foreach (var task in Repository.Query())
            {
                _tasks.Add(task);
            }
            RaisePropertyChanged(() => Tasks);

            Repository.AggregateRootChanged +=
                (o, e) =>
                    {                        
                        _tasks.Clear();
                        foreach (var task in Repository.Query())
                        {
                            _tasks.Add(task);
                        }
                        RaisePropertyChanged(() => Tasks);
                    };

            base.ActivateView(viewName, viewParameters);
        }

        protected override void InitializeVm()
        {
            EventAggregator.Subscribe<FilterBase>(this);
            EventAggregator.Subscribe<MessageNewToDoItem>(this);
            base.InitializeVm();
        }        

        public void HandleEvent(MessageNewToDoItem publishedEvent)
        {
            Repository.Save(publishedEvent.Item);
        }
    }
}