﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Xaml;

namespace ViewModelLocators
{
    public class ViewModelBinderExtension : MarkupExtension
    {
        public override object ProvideValue(
            IServiceProvider serviceProvider)
        {
            var targetProvider
                = serviceProvider
                        .GetService(typeof (IRootObjectProvider))
                    as IRootObjectProvider;

            if (targetProvider == null) return null;

            var targetControl = targetProvider.RootObject
                                as UserControl;

            if (targetControl == null) return null;

            if (targetControl is MarkupControl)
            {
                if (DesignerProperties.IsInDesignTool)
                {
                    return new DesignViewModel();
                }

                var vm
                    = new ViewModel
                            {
                                GoToVisualState =
                                    state =>
                                    VisualStateManager.GoToState(
                                        targetControl, state, true)
                            };


                return vm;
            }
            return null;
        }
    }
}