﻿using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;

namespace VisualStateManager
{
    public class VisualStateDebugBehavior : Behavior<FrameworkElement>
    {
        private const string NOTHING = "(nothing)";
        private const string ANYTHING = "(anything)";

        protected override void OnAttached()
        {
            if (!Debugger.IsAttached)
            {
                return;
            }

            AssociatedObject.Loaded += AssociatedObject_Loaded;
        }

        private void AssociatedObject_Loaded(object sender, RoutedEventArgs args)
        {
            AssociatedObject.Loaded -= AssociatedObject_Loaded;

            var groups = System.Windows.VisualStateManager
                .GetVisualStateGroups(AssociatedObject);
            foreach (var vsmGroup in
                from object @group in groups
                select @group as VisualStateGroup)
            {
                Debug.WriteLine("VSM: Found group {0}", vsmGroup.Name);
                vsmGroup.CurrentStateChanged +=
                    (o, e) =>
                    Debug
                        .WriteLine("VSM: Group {0}: state changed from {1} to {2}",
                                    vsmGroup.Name,
                                    e.OldState != null ? e.OldState.Name : NOTHING,
                                    e.NewState != null ? e.NewState.Name : NOTHING);
                foreach (var vsmState in
                    vsmGroup.States.Cast<object>()
                        .Select(state => state as VisualState))
                {
                    Debug.WriteLine("VSM: Found state {0}", vsmState.Name);

                    if (vsmState.Storyboard != null)
                    {
                        vsmState.Storyboard.Completed +=
                            (o, e) =>
                            Debug
                                .WriteLine("VSM: State {0}: storyboard completed.",
                                            vsmState.Name);
                    }
                }

                foreach (var vsmTransition in vsmGroup.Transitions.Cast<object>()
                    .Select(transition => transition as VisualTransition))
                {
                    Debug.WriteLine("VSM: Found transition from {0} to {1}",
                                    vsmTransition.From ?? ANYTHING,
                                    vsmTransition.To ?? ANYTHING);

                    if (vsmTransition.Storyboard != null)
                    {
                        vsmTransition.Storyboard.Completed +=
                            (o, e) => Debug
                                            .WriteLine(
                                                "VSM: Transition from {0} to {1}: storyboard completed.",
                                                vsmTransition.From ?? ANYTHING,
                                                vsmTransition.To ?? ANYTHING);
                    }
                }
            }
        }
    }
}