﻿using System.Collections.Generic;
using System.Windows.Controls;

namespace VsmAggregator
{
    public class VisualStateAggregator
    {
        private readonly List<VisualStateSubscription> _subscribers = new List<VisualStateSubscription>();

        public void AddSubscription(Control control, string eventName, string stateName, bool useTransitions)
        {
            _subscribers.Add(new VisualStateSubscription(control, eventName, stateName, useTransitions));
        }

        public void PublishEvent(string eventName)
        {
            var expired = new List<VisualStateSubscription>();

            foreach (var subscriber in _subscribers)
            {
                if (subscriber.IsExpired)
                {
                    expired.Add(subscriber);
                }
                else
                {
                    subscriber.RaiseEvent(eventName);
                }
            }

            expired.ForEach(s => _subscribers.Remove(s));
        }
    }

}